@@ -17,6 +17,7 @@
  * under the License.
  */
 
+#include "common/defaults.h"
 #include "common/string.h"
 #include "config.h"
 #include "resolution.h"
@@ -28,6 +29,7 @@
 #include <guacamole/client.h>
 #include <guacamole/string.h>
 #include <guacamole/user.h>
+#include <guacamole/wol-constants.h>
 #include <winpr/crt.h>
 #include <winpr/wtypes.h>
 
@@ -55,6 +57,8 @@ const char* GUAC_RDP_CLIENT_ARGS[] = {
     "drive-name",
     "drive-path",
     "create-drive-path",
+    "disable-download",
+    "disable-upload",
     "console",
     "console-audio",
     "server-layout",
@@ -91,6 +95,8 @@ const char* GUAC_RDP_CLIENT_ARGS[] = {
     "sftp-directory",
     "sftp-root-directory",
     "sftp-server-alive-interval",
+    "sftp-disable-download",
+    "sftp-disable-upload",
 #endif
 
     "recording-path",
@@ -111,6 +117,13 @@ const char* GUAC_RDP_CLIENT_ARGS[] = {
 
     "load-balance-info",
 
+    "disable-copy",
+    "disable-paste",
+    
+    "wol-send-packet",
+    "wol-mac-addr",
+    "wol-broadcast-addr",
+    "wol-wait-time",
     NULL
 };
 
@@ -212,6 +225,18 @@ enum RDP_ARGS_IDX {
      * drive if it does not yet exist, "false" or blank otherwise.
      */
     IDX_CREATE_DRIVE_PATH,
+    
+    /**
+     * "true" to disable the ability to download files from a remote server to
+     * the local client over RDP, "false" or blank otherwise.
+     */
+    IDX_DISABLE_DOWNLOAD,
+    
+    /**
+     * "true" to disable the ability to upload files from the local client to
+     * the remote server over RDP, "false" or blank otherwise.
+     */
+    IDX_DISABLE_UPLOAD,
 
     /**
      * "true" if this session is a console session, "false" or blank otherwise.
@@ -233,8 +258,8 @@ enum RDP_ARGS_IDX {
 
     /**
      * The type of security to use for the connection. Valid values are "rdp",
-     * "tls", "nla", "nla-ext", or "any". By default, the security mode is
-     * negotiated ("any").
+     * "tls", "nla", "nla-ext", "vmconnect", or "any". By default, the security
+     * mode is negotiated ("any").
      */
     IDX_SECURITY,
 
@@ -428,6 +453,20 @@ enum RDP_ARGS_IDX {
      * cases.
      */
     IDX_SFTP_SERVER_ALIVE_INTERVAL,
+    
+    /**
+     * "true" to disable file download from the SFTP server to the local client
+     * over the SFTP connection, if SFTP is configured and enabled.  "false" or
+     * blank otherwise.
+     */
+    IDX_SFTP_DISABLE_DOWNLOAD,
+    
+    /**
+     * "true" to disable file upload from the SFTP server to the local client
+     * over the SFTP connection, if SFTP is configured and enabled.  "false" or
+     * blank otherwise.
+     */
+    IDX_SFTP_DISABLE_UPLOAD,
 #endif
 
     /**
@@ -535,6 +574,49 @@ enum RDP_ARGS_IDX {
      */
     IDX_LOAD_BALANCE_INFO,
 
+    /**
+     * Whether outbound clipboard access should be blocked. If set to "true",
+     * it will not be possible to copy data from the remote desktop to the
+     * client using the clipboard. By default, clipboard access is not blocked.
+     */
+    IDX_DISABLE_COPY,
+
+    /**
+     * Whether inbound clipboard access should be blocked. If set to "true", it
+     * will not be possible to paste data from the client to the remote desktop
+     * using the clipboard. By default, clipboard access is not blocked.
+     */
+    IDX_DISABLE_PASTE,
+    
+    /**
+     * Whether or not to send the magic Wake-on-LAN (WoL) packet to the host
+     * prior to attempting to connect.  Non-zero values will enable sending
+     * the WoL packet, while zero will disable this functionality.  By default
+     * the WoL packet is not sent.
+     */
+    IDX_WOL_SEND_PACKET,
+    
+    /**
+     * The MAC address to put in the magic WoL packet for the host that should
+     * be woken up.
+     */
+    IDX_WOL_MAC_ADDR,
+    
+    /**
+     * The broadcast address to which to send the magic WoL packet to wake up
+     * the remote host.
+     */
+    IDX_WOL_BROADCAST_ADDR,
+    
+    /**
+     * The amount of time, in seconds, to wait after sending the WoL packet
+     * before attempting to connect to the host.  This should be a reasonable
+     * amount of time to allow the remote host to fully boot and respond to
+     * network connection requests.  The default is not to wait at all
+     * (0 seconds).
+     */
+    IDX_WOL_WAIT_TIME,
+
     RDP_ARGS_COUNT
 };
 
@@ -595,6 +677,12 @@ guac_rdp_settings* guac_rdp_parse_args(guac_user* user,
         settings->security_mode = GUAC_SECURITY_RDP;
     }
 
+    /* Negotiate security supported by VMConnect */
+    else if (strcmp(argv[IDX_SECURITY], "vmconnect") == 0) {
+        guac_user_log(user, GUAC_LOG_INFO, "Security mode: Hyper-V / VMConnect");
+        settings->security_mode = GUAC_SECURITY_VMCONNECT;
+    }
+
     /* Negotiate security (allow server to choose) */
     else if (strcmp(argv[IDX_SECURITY], "any") == 0) {
         guac_user_log(user, GUAC_LOG_INFO, "Security mode: Negotiate (ANY)");
@@ -612,10 +700,10 @@ guac_rdp_settings* guac_rdp_parse_args(guac_user* user,
         guac_user_parse_args_string(user, GUAC_RDP_CLIENT_ARGS, argv,
                 IDX_HOSTNAME, "");
 
-    /* If port specified, use it */
+    /* If port specified, use it, otherwise use an appropriate default */
     settings->port =
-        guac_user_parse_args_int(user, GUAC_RDP_CLIENT_ARGS, argv,
-                IDX_PORT, RDP_DEFAULT_PORT);
+        guac_user_parse_args_int(user, GUAC_RDP_CLIENT_ARGS, argv, IDX_PORT,
+                settings->security_mode == GUAC_SECURITY_VMCONNECT ? RDP_DEFAULT_VMCONNECT_PORT : RDP_DEFAULT_PORT);
 
     guac_user_log(user, GUAC_LOG_DEBUG,
             "User resolution is %ix%i at %i DPI",
@@ -792,15 +880,6 @@ guac_rdp_settings* guac_rdp_parse_args(guac_user* user,
                 "Preconnection BLOB: \"%s\"", settings->preconnection_blob);
     }
 
-    /* Warn if support for the preconnection BLOB / ID is absent */
-    if (settings->preconnection_blob != NULL
-            || settings->preconnection_id != -1) {
-        guac_user_log(user, GUAC_LOG_WARNING,
-                "Installed version of FreeRDP lacks support for the "
-                "preconnection PDU. The specified preconnection BLOB and/or "
-                "ID will be ignored.");
-    }
-
     /* Audio enable/disable */
     settings->audio_enabled =
         !guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
@@ -826,13 +905,25 @@ guac_rdp_settings* guac_rdp_parse_args(guac_user* user,
         guac_user_parse_args_string(user, GUAC_RDP_CLIENT_ARGS, argv,
                 IDX_DRIVE_NAME, "Guacamole Filesystem");
 
+    /* The path on the server to connect the drive. */
     settings->drive_path =
         guac_user_parse_args_string(user, GUAC_RDP_CLIENT_ARGS, argv,
                 IDX_DRIVE_PATH, "");
 
+    /* If the server path should be created if it doesn't already exist. */
     settings->create_drive_path =
         guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
                 IDX_CREATE_DRIVE_PATH, 0);
+    
+    /* If file downloads over RDP should be disabled. */
+    settings->disable_download =
+        guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_DISABLE_DOWNLOAD, 0);
+    
+    /* If file uploads over RDP should be disabled. */
+    settings->disable_upload =
+        guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_DISABLE_UPLOAD, 0);
 
     /* Pick keymap based on argument */
     settings->server_layout = NULL;
@@ -905,6 +996,16 @@ guac_rdp_settings* guac_rdp_parse_args(guac_user* user,
     settings->sftp_server_alive_interval =
         guac_user_parse_args_int(user, GUAC_RDP_CLIENT_ARGS, argv,
                 IDX_SFTP_SERVER_ALIVE_INTERVAL, 0);
+    
+    /* Whether or not to disable file download over SFTP. */
+    settings->sftp_disable_download =
+        guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_SFTP_DISABLE_DOWNLOAD, 0);
+    
+    /* Whether or not to disable file upload over SFTP. */
+    settings->sftp_disable_upload =
+        guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_SFTP_DISABLE_UPLOAD, 0);
 #endif
 
     /* Read recording path */
@@ -997,6 +1098,47 @@ guac_rdp_settings* guac_rdp_parse_args(guac_user* user,
         guac_user_parse_args_string(user, GUAC_RDP_CLIENT_ARGS, argv,
                 IDX_LOAD_BALANCE_INFO, NULL);
 
+    /* Parse clipboard copy disable flag */
+    settings->disable_copy =
+        guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_DISABLE_COPY, 0);
+
+    /* Parse clipboard paste disable flag */
+    settings->disable_paste =
+        guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_DISABLE_PASTE, 0);
+    
+    /* Parse Wake-on-LAN (WoL) settings */
+    settings->wol_send_packet =
+        guac_user_parse_args_boolean(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_WOL_SEND_PACKET, 0);
+    
+    if (settings->wol_send_packet) {
+        
+        /* If WoL has been requested but no MAC address given, log a warning. */
+        if(strcmp(argv[IDX_WOL_MAC_ADDR], "") == 0) {
+            guac_user_log(user, GUAC_LOG_WARNING, "WoL requested but no MAC ",
+                    "address specified.  WoL will not be sent.");
+            settings->wol_send_packet = 0;
+        }
+        
+        /* Parse the WoL MAC address. */
+        settings->wol_mac_addr = 
+            guac_user_parse_args_string(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_WOL_MAC_ADDR, NULL);
+        
+        /* Parse the WoL broadcast address. */
+        settings->wol_broadcast_addr =
+            guac_user_parse_args_string(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_WOL_BROADCAST_ADDR, GUAC_WOL_LOCAL_IPV4_BROADCAST);
+        
+        /* Parse the WoL wait time. */
+        settings->wol_wait_time =
+            guac_user_parse_args_int(user, GUAC_RDP_CLIENT_ARGS, argv,
+                IDX_WOL_WAIT_TIME, GUAC_WOL_DEFAULT_BOOT_WAIT_TIME);
+        
+    }
+
     /* Success */
     return settings;
 
@@ -1058,6 +1200,9 @@ void guac_rdp_settings_free(guac_rdp_settings* settings) {
 
     /* Free load balancer information string */
     free(settings->load_balance_info);
+    
+    free(settings->wol_mac_addr);
+    free(settings->wol_broadcast_addr);
 
     /* Free settings structure */
     free(settings);
@@ -1251,6 +1396,15 @@ void guac_rdp_push_settings(guac_client* client,
             rdp_settings->ExtSecurity = TRUE;
             break;
 
+        /* Hyper-V "VMConnect" negotiation mode */
+        case GUAC_SECURITY_VMCONNECT:
+            rdp_settings->RdpSecurity = FALSE;
+            rdp_settings->TlsSecurity = TRUE;
+            rdp_settings->NlaSecurity = TRUE;
+            rdp_settings->ExtSecurity = FALSE;
+            rdp_settings->VmConnectMode = TRUE;
+            break;
+
         /* All security types */
         case GUAC_SECURITY_ANY:
             rdp_settings->RdpSecurity = TRUE;
@@ -1270,9 +1424,9 @@ void guac_rdp_push_settings(guac_client* client,
         rdp_settings->Workarea = TRUE;
         rdp_settings->RemoteApplicationMode = TRUE;
         rdp_settings->RemoteAppLanguageBarSupported = TRUE;
-        rdp_settings->RemoteApplicationProgram = guac_settings->remote_app;
+        rdp_settings->RemoteApplicationProgram = guac_rdp_strdup(guac_settings->remote_app);
         rdp_settings->ShellWorkingDirectory = guac_rdp_strdup(guac_settings->remote_app_dir);
-        rdp_settings->RemoteApplicationCmdLine = guac_settings->remote_app_args;
+        rdp_settings->RemoteApplicationCmdLine = guac_rdp_strdup(guac_settings->remote_app_args);
     }
 
     /* Preconnection ID */
@@ -1286,7 +1440,7 @@ void guac_rdp_push_settings(guac_client* client,
     if (guac_settings->preconnection_blob != NULL) {
         rdp_settings->NegotiateSecurityLayer = FALSE;
         rdp_settings->SendPreconnectionPdu = TRUE;
-        rdp_settings->PreconnectionBlob = guac_settings->preconnection_blob;
+        rdp_settings->PreconnectionBlob = guac_rdp_strdup(guac_settings->preconnection_blob);
     }
 
     /* Enable use of RD gateway if a gateway hostname is provided */
@@ -1330,5 +1484,10 @@ void guac_rdp_push_settings(guac_client* client,
     rdp_settings->OrderSupport[NEG_FAST_INDEX_INDEX] = !guac_settings->disable_glyph_caching;
     rdp_settings->OrderSupport[NEG_FAST_GLYPH_INDEX] = !guac_settings->disable_glyph_caching;
 
+#ifdef HAVE_RDPSETTINGS_ALLOWUNANOUNCEDORDERSFROMSERVER
+    /* Do not consider server use of unannounced orders to be a fatal error */
+    rdp_settings->AllowUnanouncedOrdersFromServer = TRUE;
+#endif
+
 }
 
