@@ -19,9 +19,11 @@
 
 #include "config.h"
 
+#include "common/defaults.h"
 #include "settings.h"
 
 #include <guacamole/user.h>
+#include <guacamole/wol-constants.h>
 
 #include <sys/types.h>
 #include <regex.h>
@@ -55,6 +57,12 @@ const char* GUAC_TELNET_CLIENT_ARGS[] = {
     "scrollback",
     "login-success-regex",
     "login-failure-regex",
+    "disable-copy",
+    "disable-paste",
+    "wol-send-packet",
+    "wol-mac-addr",
+    "wol-broadcast-addr",
+    "wol-wait-time",
     NULL
 };
 
@@ -216,6 +224,46 @@ enum TELNET_ARGS_IDX {
      */
     IDX_LOGIN_FAILURE_REGEX,
 
+    /**
+     * Whether outbound clipboard access should be blocked. If set to "true",
+     * it will not be possible to copy data from the terminal to the client
+     * using the clipboard. By default, clipboard access is not blocked.
+     */
+    IDX_DISABLE_COPY,
+
+    /**
+     * Whether inbound clipboard access should be blocked. If set to "true", it
+     * will not be possible to paste data from the client to the terminal using
+     * the clipboard. By default, clipboard access is not blocked.
+     */
+    IDX_DISABLE_PASTE,
+    
+    /**
+     * Whether to send the magic Wake-on-LAN (WoL) packet.  If set to "true"
+     * the WoL packet will be sent prior to attempting to connect to the remote
+     * host.  By default the WoL packet will not be sent.
+     */
+    IDX_WOL_SEND_PACKET,
+    
+    /**
+     * The MAC address to put in the magic WoL packet to wake the remote host.
+     * There is no default value, and if WoL is enabled but the MAC address
+     * is not provided, a warning will be logged and no packet will be sent.
+     */
+    IDX_WOL_MAC_ADDR,
+    
+    /**
+     * The broadcast address to which to send the magic WoL packet.
+     */
+    IDX_WOL_BROADCAST_ADDR,
+    
+    /**
+     * The amount of time, in seconds, to wait after the magic WoL packet is
+     * sent before continuing the connection attempt.  The default is not to
+     * wait at all (0 seconds).
+     */
+    IDX_WOL_WAIT_TIME,
+
     TELNET_ARGS_COUNT
 };
 
@@ -428,6 +476,47 @@ guac_telnet_settings* guac_telnet_parse_args(guac_user* user,
         guac_user_parse_args_string(user, GUAC_TELNET_CLIENT_ARGS, argv,
                 IDX_TERMINAL_TYPE, "linux");
 
+    /* Parse clipboard copy disable flag */
+    settings->disable_copy =
+        guac_user_parse_args_boolean(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_DISABLE_COPY, false);
+
+    /* Parse clipboard paste disable flag */
+    settings->disable_paste =
+        guac_user_parse_args_boolean(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_DISABLE_PASTE, false);
+    
+    /* Parse Wake-on-LAN (WoL) settings */
+    settings->wol_send_packet =
+        guac_user_parse_args_boolean(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_WOL_SEND_PACKET, false);
+    
+    if (settings->wol_send_packet) {
+        
+        /* If WoL has been enabled but no MAC provided, log warning and disable. */
+        if(strcmp(argv[IDX_WOL_MAC_ADDR], "") == 0) {
+            guac_user_log(user, GUAC_LOG_WARNING, "Wake on LAN was requested, ",
+                    "but no MAC address was specified.  WoL will not be sent.");
+            settings->wol_send_packet = false;
+        }
+        
+        /* Parse the WoL MAC address. */
+        settings->wol_mac_addr =
+            guac_user_parse_args_string(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_WOL_MAC_ADDR, NULL);
+        
+        /* Parse the WoL broadcast address. */
+        settings->wol_broadcast_addr =
+            guac_user_parse_args_string(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_WOL_BROADCAST_ADDR, GUAC_WOL_LOCAL_IPV4_BROADCAST);
+        
+        /* Parse the WoL wait time. */
+        settings->wol_wait_time =
+            guac_user_parse_args_int(user, GUAC_TELNET_CLIENT_ARGS, argv,
+                IDX_WOL_WAIT_TIME, GUAC_WOL_DEFAULT_BOOT_WAIT_TIME);
+        
+    }
+
     /* Parsing was successful */
     return settings;
 
